Status

Enum Status 

pub enum Status {
    InvalidArgument,
    FailedPrecondition,
    PermissionDenied,
    NotFound,
    ResourceExhausted,
    Internal,
    Unavailable,
    DeadlineExceeded,
}

Variants



InvalidArgument

The request body is malformed.



FailedPrecondition

Gemini API free tier is not available in your country. Please enable billing on your project in Google AI Studio.



PermissionDenied

Your API key doesn’t have the required permissions.



NotFound

The requested resource wasn’t found.



ResourceExhausted

You’ve exceeded the rate limit.



Internal

An unexpected error occurred on Google’s side.



Unavailable

The service may be temporarily overloaded or down.



DeadlineExceeded

The service is unable to finish processing within the deadline.